@@ -10,6 +10,7 @@ use dprint_cli_core::logging::Logger;
 use dprint_cli_core::logging::LoggerOptions;
 use dprint_cli_core::logging::ProgressBars;
 use std::fs;
+use std::num::NonZeroUsize;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
@@ -238,8 +239,8 @@ impl Environment for RealEnvironment {
     std::env::consts::OS.to_string()
   }
 
-  fn available_parallelism(&self) -> usize {
-    std::thread::available_parallelism().map(|p| p.get()).unwrap_or(4)
+  fn max_threads(&self) -> usize {
+    resolve_max_threads(std::env::var("DPRINT_MAX_THREADS").ok(), std::thread::available_parallelism().ok())
   }
 
   fn cli_version(&self) -> String {
@@ -295,7 +296,6 @@ impl Environment for RealEnvironment {
 
   #[cfg(windows)]
   fn ensure_system_path(&self, directory_path: &str) -> Result<()> {
-    // from bvm (https://github.com/bvm/bvm)
     use winreg::enums::*;
     use winreg::RegKey;
     log_verbose!(self, "Ensuring '{}' is on the path.", directory_path);
@@ -317,7 +317,6 @@ impl Environment for RealEnvironment {
 
   #[cfg(windows)]
   fn remove_system_path(&self, directory_path: &str) -> Result<()> {
-    // from bvm (https://github.com/bvm/bvm)
     use winreg::enums::*;
     use winreg::RegKey;
     log_verbose!(self, "Ensuring '{}' is on the path.", directory_path);
@@ -338,6 +337,26 @@ impl Environment for RealEnvironment {
   }
 }
 
+fn resolve_max_threads(env_var: Option<String>, available_parallelism: Option<NonZeroUsize>) -> usize {
+  fn maybe_specified_threads(env_var: Option<String>) -> Option<usize> {
+    let value = env_var?.parse::<usize>().ok()?;
+    if value > 0 {
+      Some(value)
+    } else {
+      None
+    }
+  }
+
+  let maybe_actual_count = available_parallelism.map(|p| p.get());
+  match maybe_specified_threads(env_var) {
+    Some(specified_count) => match maybe_actual_count {
+      Some(actual_count) if specified_count > actual_count => actual_count,
+      _ => specified_count,
+    },
+    _ => maybe_actual_count.unwrap_or(4),
+  }
+}
+
 const CACHE_DIR_ENV_VAR_NAME: &str = "DPRINT_CACHE_DIR";
 
 fn get_cache_dir() -> Result<PathBuf> {
@@ -396,4 +415,15 @@ mod test {
       "The DPRINT_CACHE_DIR environment variable must specify an absolute path."
     );
   }
+
+  #[test]
+  fn should_resolve_num_threads() {
+    assert_eq!(resolve_max_threads(None, None), 4);
+    assert_eq!(resolve_max_threads(None, NonZeroUsize::new(1)), 1);
+    assert_eq!(resolve_max_threads(None, NonZeroUsize::new(4)), 4);
+    assert_eq!(resolve_max_threads(Some("2".to_string()), NonZeroUsize::new(4)), 2);
+    assert_eq!(resolve_max_threads(Some("0".to_string()), NonZeroUsize::new(4)), 4);
+    assert_eq!(resolve_max_threads(Some("5".to_string()), NonZeroUsize::new(4)), 4);
+    assert_eq!(resolve_max_threads(Some("4".to_string()), NonZeroUsize::new(4)), 4);
+  }
 }
